@@ -30,6 +30,10 @@
 #include "apr_hash.h"
 #include "apr_optional.h"
 #include "util_filter.h"
+#include "ap_expr.h"
+#include "apr_tables.h"
+
+#include "http_config.h"
 
 #if APR_HAVE_STRUCT_RLIMIT
 #include <sys/time.h>
@@ -73,31 +77,18 @@ extern "C" {
 #define OPT_EXECCGI 8
 /**  directive unset */
 #define OPT_UNSET 16
-/**  IncludesNOEXEC directive */
-#define OPT_INCNOEXEC 32
+/**  SSI exec= permission is permitted, iff OPT_INCLUDES is also set */
+#define OPT_INC_WITH_EXEC 32
 /** SymLinksIfOwnerMatch directive */
 #define OPT_SYM_OWNER 64
 /** MultiViews directive */
 #define OPT_MULTI 128
 /**  All directives */
-#define OPT_ALL (OPT_INDEXES|OPT_INCLUDES|OPT_INCNOEXEC|OPT_SYM_LINKS|OPT_EXECCGI)
+#define OPT_ALL (OPT_INDEXES|OPT_INCLUDES|OPT_INC_WITH_EXEC|OPT_SYM_LINKS|OPT_EXECCGI)
 /** @} */
 
-#ifdef CORE_PRIVATE
-/* For internal use only - since 2.2.12, the OPT_INCNOEXEC bit is
- * internally replaced by OPT_INC_WITH_EXEC.  The internal semantics
- * of the two SSI-related bits are hence:
- *
- *  OPT_INCLUDES => "enable SSI, without exec= permission"
- *  OPT_INC_WITH_EXEC => "iff OPT_INCLUDES is set, also enable exec="
- *
- * The set of options exposed via ap_allow_options() retains the
- * semantics of OPT_INCNOEXEC by flipping the bit. */
-#define OPT_INC_WITH_EXEC OPT_INCNOEXEC
-#endif
-
 /**
- * @defgroup get_remote_host Remote Host Resolution 
+ * @defgroup get_remote_host Remote Host Resolution
  * @ingroup APACHE_CORE_HTTPD
  * @{
  */
@@ -158,17 +149,10 @@ AP_DECLARE(int) ap_allow_options(request_rec *r);
  */
 AP_DECLARE(int) ap_allow_overrides(request_rec *r);
 
-/**
- * Retrieve the value of the DefaultType directive, or text/plain if not set
- * @param r The current request
- * @return The default type
- */
-AP_DECLARE(const char *) ap_default_type(request_rec *r);     
-
 /**
  * Retrieve the document root for this server
  * @param r The current request
- * @warning Don't use this!  If your request went through a Userdir, or 
+ * @warning Don't use this!  If your request went through a Userdir, or
  * something like that, it'll screw you.  But it's back-compatible...
  * @return The document root
  */
@@ -191,10 +175,10 @@ AP_DECLARE(const char *) ap_document_root(request_rec *r);
  *                     never forced.
  *     REMOTE_DOUBLE_REV will always force a DNS lookup, and also force
  *                   a double reverse lookup, regardless of the HostnameLookups
- *                   setting.  The result is the (double reverse checked) 
+ *                   setting.  The result is the (double reverse checked)
  *                   hostname, or NULL if any of the lookups fail.
  * </pre>
- * @param str_is_ip unless NULL is passed, this will be set to non-zero on output when an IP address 
+ * @param str_is_ip unless NULL is passed, this will be set to non-zero on output when an IP address
  *        string is returned
  * @return The remote hostname
  */
@@ -227,6 +211,15 @@ AP_DECLARE(char *) ap_construct_url(apr_pool_t *p, const char *uri, request_rec
  */
 AP_DECLARE(const char *) ap_get_server_name(request_rec *r);
 
+/**
+ * Get the current server name from the request for the purposes
+ * of using in a URL.  If the server name is an IPv6 literal
+ * address, it will be returned in URL format (e.g., "[fe80::1]").
+ * @param r The current request
+ * @return the server name
+ */
+AP_DECLARE(const char *) ap_get_server_name_for_url(request_rec *r);
+
 /**
  * Get the current server port
  * @param r The current request
@@ -235,7 +228,7 @@ AP_DECLARE(const char *) ap_get_server_name(request_rec *r);
 AP_DECLARE(apr_port_t) ap_get_server_port(const request_rec *r);
 
 /**
- * Return the limit on bytes in request msg body 
+ * Return the limit on bytes in request msg body
  * @param r The current request
  * @return the maximum number of bytes in the request msg body
  */
@@ -246,7 +239,7 @@ AP_DECLARE(apr_off_t) ap_get_limit_req_body(const request_rec *r);
  * @param r The current request
  * @return the maximum number of bytes in XML request msg body
  */
-AP_DECLARE(size_t) ap_get_limit_xml_body(const request_rec *r);
+AP_DECLARE(apr_size_t) ap_get_limit_xml_body(const request_rec *r);
 
 /**
  * Install a custom response handler for a given status
@@ -283,8 +276,8 @@ AP_DECLARE_NONSTD(int) ap_core_translate(request_rec *r);
 /** @see require_line */
 typedef struct require_line require_line;
 
-/** 
- * @brief A structure to keep track of authorization requirements 
+/**
+ * @brief A structure to keep track of authorization requirements
 */
 struct require_line {
     /** Where the require line is in the config file. */
@@ -292,7 +285,7 @@ struct require_line {
     /** The complete string from the command line */
     char *requirement;
 };
-     
+
 /**
  * Return the type of authorization required for this request
  * @param r The current request
@@ -305,7 +298,7 @@ AP_DECLARE(const char *) ap_auth_type(request_rec *r);
  * @param r The current request
  * @return The current authorization realm
  */
-AP_DECLARE(const char *) ap_auth_name(request_rec *r);     
+AP_DECLARE(const char *) ap_auth_name(request_rec *r);
 
 /**
  * How the requires lines must be met.
@@ -319,15 +312,6 @@ AP_DECLARE(const char *) ap_auth_name(request_rec *r);
  */
 AP_DECLARE(int) ap_satisfies(request_rec *r);
 
-/**
- * Retrieve information about all of the requires directives for this request
- * @param r The current request
- * @return An array of all requires directives for this request
- */
-AP_DECLARE(const apr_array_header_t *) ap_requires(request_rec *r);    
-
-#ifdef CORE_PRIVATE
-
 /**
  * Core is also unlike other modules in being implemented in more than
  * one file... so, data structures are declared here, even though most of
@@ -336,10 +320,45 @@ AP_DECLARE(const apr_array_header_t *) ap_requires(request_rec *r);
 AP_DECLARE_DATA extern module core_module;
 
 /**
- * @brief  Per-request configuration 
+ * Accessor for core_module's specific data. Equivalent to
+ * ap_get_module_config(cv, &core_module) but more efficient.
+ * @param cv The vector in which the modules configuration is stored.
+ *        usually r->per_dir_config or s->module_config
+ * @return The module-specific data
+ */
+AP_DECLARE(void *) ap_get_core_module_config(const ap_conf_vector_t *cv);
+
+/**
+ * Accessor to set core_module's specific data. Equivalent to
+ * ap_set_module_config(cv, &core_module, val) but more efficient.
+ * @param cv The vector in which the modules configuration is stored.
+ *        usually r->per_dir_config or s->module_config
+ * @param val The module-specific data to set
+ */
+AP_DECLARE(void) ap_set_core_module_config(ap_conf_vector_t *cv, void *val);
+
+/** Get the socket from the core network filter. This should be used instead of
+ * accessing the core connection config directly.
+ * @param c The connection record
+ * @return The socket
+ */
+AP_DECLARE(apr_socket_t *) ap_get_conn_socket(conn_rec *c);
+
+#ifndef AP_DEBUG
+#define AP_CORE_MODULE_INDEX  0
+#define ap_get_core_module_config(v) \
+    (((void **)(v))[AP_CORE_MODULE_INDEX])
+#define ap_set_core_module_config(v, val) \
+    ((((void **)(v))[AP_CORE_MODULE_INDEX]) = (val))
+#else
+#define AP_CORE_MODULE_INDEX  (AP_DEBUG_ASSERT(core_module.module_index == 0), 0)
+#endif
+
+/**
+ * @brief  Per-request configuration
 */
 typedef struct {
-    /** bucket brigade used by getline for look-ahead and 
+    /** bucket brigade used by getline for look-ahead and
      * ap_get_client_block for holding left-over request body */
     struct apr_bucket_brigade *bb;
 
@@ -350,6 +369,31 @@ typedef struct {
      */
     void **notes;
 
+    /** Custom response strings registered via ap_custom_response(),
+     * or NULL; check per-dir config if nothing found here
+     */
+    char **response_code_strings; /* from ap_custom_response(), not from
+                                   * ErrorDocument
+                                   */
+
+    /** per-request document root of the server. This allows mass vhosting
+     * modules better compatibility with some scripts. Normally the
+     * context_* info should be used instead */
+    const char *document_root;
+
+    /*
+     * more fine-grained context information which is set by modules like
+     * mod_alias and mod_userdir
+     */
+    /** the context root directory on disk for the current resource,
+     *  without trailing slash
+     */
+    const char *context_document_root;
+    /** the URI prefix that corresponds to the context_document_root directory,
+     *  without trailing slash
+     */
+    const char *context_prefix;
+
     /** There is a script processor installed on the output filter chain,
      * so it needs the default_handler to deliver a (script) file into
      * the chain so it can process it. Normally, default_handler only
@@ -360,12 +404,6 @@ typedef struct {
      */
     int deliver_script;
 
-    /** Custom response strings registered via ap_custom_response(),
-     * or NULL; check per-dir config if nothing found here
-     */
-    char **response_code_strings; /* from ap_custom_response(), not from
-                                   * ErrorDocument
-                                   */
     /** Should addition of charset= be suppressed for this request?
      */
     int suppress_charset;
@@ -378,7 +416,8 @@ typedef struct {
 #define AP_NOTE_DIRECTORY_WALK 0
 #define AP_NOTE_LOCATION_WALK  1
 #define AP_NOTE_FILE_WALK      2
-#define AP_NUM_STD_NOTES       3
+#define AP_NOTE_IF_WALK        3
+#define AP_NUM_STD_NOTES       4
 
 /**
  * Reserve an element in the core_request_config->notes array
@@ -407,7 +446,7 @@ AP_DECLARE(void **) ap_get_request_note(request_rec *r, apr_size_t note_num);
 
 
 typedef unsigned char allow_options_t;
-typedef unsigned char overrides_t;
+typedef unsigned int overrides_t;
 
 /*
  * Bits of info that go into making an ETag for a file
@@ -422,8 +461,9 @@ typedef unsigned long etag_components_t;
 #define ETAG_MTIME (1 << 1)
 #define ETAG_INODE (1 << 2)
 #define ETAG_SIZE  (1 << 3)
-#define ETAG_BACKWARD (ETAG_MTIME | ETAG_INODE | ETAG_SIZE)
 #define ETAG_ALL   (ETAG_MTIME | ETAG_INODE | ETAG_SIZE)
+/* This is the default value used */
+#define ETAG_BACKWARD (ETAG_MTIME | ETAG_SIZE)
 
 /**
  * @brief Server Signature Enumeration
@@ -435,8 +475,8 @@ typedef enum {
     srv_sig_withmail
 } server_signature_e;
 
-/** 
- * @brief Per-directory configuration 
+/**
+ * @brief Per-directory configuration
  */
 typedef struct {
     /** path of the directory/regex/etc. see also d_is_fnmatch/absolute below */
@@ -456,38 +496,19 @@ typedef struct {
     allow_options_t opts_remove;
     overrides_t override;
     allow_options_t override_opts;
-    
-    /* MIME typing --- the core doesn't do anything at all with this,
-     * but it does know what to slap on a request for a document which
-     * goes untyped by other mechanisms before it slips out the door...
-     */
-    
-    char *ap_default_type;
-  
-    /* Authentication stuff.  Groan... */
-    
-    int *satisfy; /* for every method one */
-    char *ap_auth_type;
-    char *ap_auth_name;
-    apr_array_header_t *ap_requires;
 
-    /* Custom response config. These can contain text or a URL to redirect to.
-     * if response_code_strings is NULL then there are none in the config,
-     * if it's not null then it's allocated to sizeof(char*)*RESPONSE_CODES.
-     * This lets us do quick merges in merge_core_dir_configs().
-     */
-  
+    /* Used to be the custom response config. No longer used. */
     char **response_code_strings; /* from ErrorDocument, not from
                                    * ap_custom_response() */
 
     /* Hostname resolution etc */
-#define HOSTNAME_LOOKUP_OFF	0
-#define HOSTNAME_LOOKUP_ON	1
-#define HOSTNAME_LOOKUP_DOUBLE	2
-#define HOSTNAME_LOOKUP_UNSET	3
+#define HOSTNAME_LOOKUP_OFF     0
+#define HOSTNAME_LOOKUP_ON      1
+#define HOSTNAME_LOOKUP_DOUBLE  2
+#define HOSTNAME_LOOKUP_UNSET   3
     unsigned int hostname_lookups : 4;
 
-    signed int content_md5 : 2;  /* calculate Content-MD5? */
+    unsigned int content_md5 : 2;  /* calculate Content-MD5? */
 
 #define USE_CANONICAL_NAME_OFF   (0)
 #define USE_CANONICAL_NAME_ON    (1)
@@ -527,10 +548,9 @@ typedef struct {
 
     server_signature_e server_signature;
 
-    int loglevel;
-    
     /* Access control */
     apr_array_header_t *sec_file;
+    apr_array_header_t *sec_if;
     ap_regex_t *r;
 
     const char *mime_type;       /* forced with ForceType  */
@@ -539,8 +559,6 @@ typedef struct {
     const char *input_filters;   /* forced with SetInputFilters */
     int accept_path_info;        /* forced with AcceptPathInfo */
 
-    apr_hash_t *ct_output_filters; /* added with AddOutputFilterByType */
-
     /*
      * What attributes/data should be included in ETag generation?
      */
@@ -559,41 +577,77 @@ typedef struct {
 #define ENABLE_SENDFILE_OFF    (0)
 #define ENABLE_SENDFILE_ON     (1)
 #define ENABLE_SENDFILE_UNSET  (2)
-    unsigned int enable_sendfile : 2;  /* files in this dir can be mmap'ed */
-    unsigned int allow_encoded_slashes : 1; /* URLs may contain %2f w/o being
-                                             * pitched indiscriminately */
+    unsigned int enable_sendfile : 2;  /* files in this dir can be sendfile'ed */
 
 #define USE_CANONICAL_PHYS_PORT_OFF   (0)
 #define USE_CANONICAL_PHYS_PORT_ON    (1)
 #define USE_CANONICAL_PHYS_PORT_UNSET (2)
-    unsigned use_canonical_phys_port : 2;
-
+    unsigned int use_canonical_phys_port : 2;
 
+    unsigned int allow_encoded_slashes : 1; /* URLs may contain %2f w/o being
+                                             * pitched indiscriminately */
     unsigned int decode_encoded_slashes : 1; /* whether to decode encoded slashes in URLs */
 
+#define AP_CONDITION_IF        1
+#define AP_CONDITION_ELSE      2
+#define AP_CONDITION_ELSEIF    (AP_CONDITION_ELSE|AP_CONDITION_IF)
+    unsigned int condition_ifelse : 2; /* is this an <If>, <ElseIf>, or <Else> */
+
+    ap_expr_info_t *condition;   /* Conditionally merge <If> sections */
+
+    /** per-dir log config */
+    struct ap_logconf *log;
+
+    /** Table of directives allowed per AllowOverrideList */
+    apr_table_t *override_list;
+
 #define AP_MAXRANGES_UNSET     -1
 #define AP_MAXRANGES_DEFAULT   -2
 #define AP_MAXRANGES_UNLIMITED -3
 #define AP_MAXRANGES_NORANGES   0
     /** Number of Ranges before returning HTTP_OK. **/
     int max_ranges;
+    /** Max number of Range overlaps (merges) allowed **/
+    int max_overlaps;
+    /** Max number of Range reversals (eg: 200-300, 100-125) allowed **/
+    int max_reversals;
 
+    /** Named back references */
+    apr_array_header_t *refs;
+
+    /** Custom response config with expression support. The hash table
+     * contains compiled expressions keyed against the custom response
+     * code.
+     */
+    apr_hash_t *response_code_exprs;
+
+#define AP_CGI_PASS_AUTH_OFF     (0)
+#define AP_CGI_PASS_AUTH_ON      (1)
+#define AP_CGI_PASS_AUTH_UNSET   (2)
+    /** CGIPassAuth: Whether HTTP authorization headers will be passed to
+     * scripts as CGI variables; affects all modules calling
+     * ap_add_common_vars(), as well as any others using this field as 
+     * advice
+     */
+    unsigned int cgi_pass_auth : 2;
 } core_dir_config;
 
+/* macro to implement off by default behaviour */
+#define AP_SENDFILE_ENABLED(x) \
+    ((x) == ENABLE_SENDFILE_ON ? APR_SENDFILE_ENABLED : 0)
+
 /* Per-server core configuration */
 
 typedef struct {
-  
-#ifdef GPROF
+
     char *gprof_dir;
-#endif
 
     /* Name translations --- we want the core to be able to do *something*
      * so it's at least a minimally functional web server on its own (and
      * can be tested that way).  But let's keep it to the bare minimum:
      */
     const char *ap_document_root;
-  
+
     /* Access control */
 
     char *access_name;
@@ -607,12 +661,25 @@ typedef struct {
     const char *protocol;
     apr_table_t *accf_map;
 
+    /* array of ap_errorlog_format_item for error log format string */
+    apr_array_header_t *error_log_format;
+    /*
+     * two arrays of arrays of ap_errorlog_format_item for additional information
+     * logged to the error log once per connection/request
+     */
+    apr_array_header_t *error_log_conn;
+    apr_array_header_t *error_log_req;
+
     /* TRACE control */
 #define AP_TRACE_UNSET    -1
 #define AP_TRACE_DISABLE   0
 #define AP_TRACE_ENABLE    1
 #define AP_TRACE_EXTENDED  2
     int trace_enable;
+#define AP_MERGE_TRAILERS_UNSET    0
+#define AP_MERGE_TRAILERS_ENABLE   1
+#define AP_MERGE_TRAILERS_DISABLE  2
+    int merge_trailers;
 
 } core_server_config;
 
@@ -625,20 +692,47 @@ void ap_core_reorder_directories(apr_pool_t *, server_rec *);
 /* for mod_perl */
 AP_CORE_DECLARE(void) ap_add_per_dir_conf(server_rec *s, void *dir_config);
 AP_CORE_DECLARE(void) ap_add_per_url_conf(server_rec *s, void *url_config);
-AP_CORE_DECLARE(void) ap_add_file_conf(core_dir_config *conf, void *url_config);
+AP_CORE_DECLARE(void) ap_add_file_conf(apr_pool_t *p, core_dir_config *conf, void *url_config);
+AP_CORE_DECLARE(const char *) ap_add_if_conf(apr_pool_t *p, core_dir_config *conf, void *url_config);
 AP_CORE_DECLARE_NONSTD(const char *) ap_limit_section(cmd_parms *cmd, void *dummy, const char *arg);
 
 /* Core filters; not exported. */
-int ap_core_input_filter(ap_filter_t *f, apr_bucket_brigade *b,
-                         ap_input_mode_t mode, apr_read_type_e block,
-                         apr_off_t readbytes);
+apr_status_t ap_core_input_filter(ap_filter_t *f, apr_bucket_brigade *b,
+                                  ap_input_mode_t mode, apr_read_type_e block,
+                                  apr_off_t readbytes);
 apr_status_t ap_core_output_filter(ap_filter_t *f, apr_bucket_brigade *b);
 
-#endif /* CORE_PRIVATE */
 
 AP_DECLARE(const char*) ap_get_server_protocol(server_rec* s);
 AP_DECLARE(void) ap_set_server_protocol(server_rec* s, const char* proto);
 
+typedef struct core_output_filter_ctx core_output_filter_ctx_t;
+typedef struct core_filter_ctx        core_ctx_t;
+
+typedef struct core_net_rec {
+    /** Connection to the client */
+    apr_socket_t *client_socket;
+
+    /** connection record */
+    conn_rec *c;
+
+    core_output_filter_ctx_t *out_ctx;
+    core_ctx_t *in_ctx;
+} core_net_rec;
+
+/**
+ * Insert the network bucket into the core input filter's input brigade.
+ * This hook is intended for MPMs or protocol modules that need to do special
+ * socket setup.
+ * @param c The connection
+ * @param bb The brigade to insert the bucket into
+ * @param socket The socket to put into a bucket
+ * @return AP_DECLINED if the current function does not handle this connection,
+ *         APR_SUCCESS or an error otherwise.
+ */
+AP_DECLARE_HOOK(apr_status_t, insert_network_bucket,
+                (conn_rec *c, apr_bucket_brigade *bb, apr_socket_t *socket))
+
 /* ----------------------------------------------------------------------
  *
  * Runtime status/management
@@ -664,10 +758,10 @@ typedef struct {
 } ap_mgmt_item_t;
 
 /* Handles for core filters */
-extern AP_DECLARE_DATA ap_filter_rec_t *ap_subreq_core_filter_handle;
-extern AP_DECLARE_DATA ap_filter_rec_t *ap_core_output_filter_handle;
-extern AP_DECLARE_DATA ap_filter_rec_t *ap_content_length_filter_handle;
-extern AP_DECLARE_DATA ap_filter_rec_t *ap_core_input_filter_handle;
+AP_DECLARE_DATA extern ap_filter_rec_t *ap_subreq_core_filter_handle;
+AP_DECLARE_DATA extern ap_filter_rec_t *ap_core_output_filter_handle;
+AP_DECLARE_DATA extern ap_filter_rec_t *ap_content_length_filter_handle;
+AP_DECLARE_DATA extern ap_filter_rec_t *ap_core_input_filter_handle;
 
 /**
  * This hook provdes a way for modules to provide metrics/statistics about
@@ -698,6 +792,121 @@ APR_DECLARE_OPTIONAL_FN(void, ap_logio_add_bytes_out,
 APR_DECLARE_OPTIONAL_FN(void, ap_logio_add_bytes_in,
                         (conn_rec *c, apr_off_t bytes));
 
+APR_DECLARE_OPTIONAL_FN(apr_off_t, ap_logio_get_last_bytes, (conn_rec *c));
+
+/* ----------------------------------------------------------------------
+ *
+ * Error log formats
+ */
+
+/**
+ * The info structure passed to callback functions of errorlog handlers.
+ * Not all information is available in all contexts. In particular, all
+ * pointers may be NULL.
+ */
+typedef struct ap_errorlog_info {
+    /** current server_rec.
+     *  Should be preferred over c->base_server and r->server
+     */
+    const server_rec *s;
+
+    /** current conn_rec.
+     *  Should be preferred over r->connection
+     */
+    const conn_rec *c;
+
+    /** current request_rec. */
+    const request_rec *r;
+    /** r->main if r is a subrequest, otherwise equal to r */
+    const request_rec *rmain;
+
+    /** pool passed to ap_log_perror, NULL otherwise */
+    apr_pool_t *pool;
+
+    /** name of source file where the log message was produced, NULL if N/A. */
+    const char *file;
+    /** line number in the source file, 0 if N/A */
+    int line;
+
+    /** module index of module that produced the log message, APLOG_NO_MODULE if N/A. */
+    int module_index;
+    /** log level of error message (flags like APLOG_STARTUP have been removed), -1 if N/A */
+    int level;
+
+    /** apr error status related to the log message, 0 if no error */
+    apr_status_t status;
+
+    /** 1 if logging to syslog, 0 otherwise */
+    int using_syslog;
+    /** 1 if APLOG_STARTUP was set for the log message, 0 otherwise */
+    int startup;
+
+    /** message format */
+    const char *format;
+} ap_errorlog_info;
+
+/**
+ * callback function prototype for a external errorlog handler
+ * @note To avoid unbounded memory usage, these functions must not allocate
+ * memory from the server, connection, or request pools. If an errorlog
+ * handler absolutely needs a pool to pass to other functions, it must create
+ * and destroy a sub-pool.
+ */
+typedef int ap_errorlog_handler_fn_t(const ap_errorlog_info *info,
+                                     const char *arg, char *buf, int buflen);
+
+/**
+ * Register external errorlog handler
+ * @param p config pool to use
+ * @param tag the new format specifier (i.e. the letter after the %)
+ * @param handler the handler function
+ * @param flags flags (reserved, set to 0)
+ */
+AP_DECLARE(void) ap_register_errorlog_handler(apr_pool_t *p, char *tag,
+                                              ap_errorlog_handler_fn_t *handler,
+                                              int flags);
+
+typedef struct ap_errorlog_handler {
+    ap_errorlog_handler_fn_t *func;
+    int flags; /* for future extensions */
+} ap_errorlog_handler;
+
+  /** item starts a new field */
+#define AP_ERRORLOG_FLAG_FIELD_SEP       1
+  /** item is the actual error message */
+#define AP_ERRORLOG_FLAG_MESSAGE         2
+  /** skip whole line if item is zero-length */
+#define AP_ERRORLOG_FLAG_REQUIRED        4
+  /** log zero-length item as '-' */
+#define AP_ERRORLOG_FLAG_NULL_AS_HYPHEN  8
+
+typedef struct {
+    /** ap_errorlog_handler function */
+    ap_errorlog_handler_fn_t *func;
+    /** argument passed to item in {} */
+    const char *arg;
+    /** a combination of the AP_ERRORLOG_* flags */
+    unsigned int flags;
+    /** only log item if the message's log level is higher than this */
+    unsigned int min_loglevel;
+} ap_errorlog_format_item;
+
+/**
+ * hook method to log error messages
+ * @ingroup hooks
+ * @param info pointer to ap_errorlog_info struct which contains all
+ *        the details
+ * @param errstr the (unformatted) message to log
+ * @warning Allocating from the usual pools (pool, info->c->pool, info->p->pool)
+ *          must be avoided because it can cause memory leaks.
+ *          Use a subpool if necessary.
+ */
+AP_DECLARE_HOOK(void, error_log, (const ap_errorlog_info *info,
+                                  const char *errstr))
+
+AP_CORE_DECLARE(void) ap_register_log_hooks(apr_pool_t *p);
+AP_CORE_DECLARE(void) ap_register_config_hooks(apr_pool_t *p);
+
 /* ----------------------------------------------------------------------
  *
  * ident lookups with mod_ident
@@ -706,11 +915,75 @@ APR_DECLARE_OPTIONAL_FN(void, ap_logio_add_bytes_in,
 APR_DECLARE_OPTIONAL_FN(const char *, ap_ident_lookup,
                         (request_rec *r));
 
+/* ----------------------------------------------------------------------
+ *
+ * authorization values with mod_authz_core
+ */
+
+APR_DECLARE_OPTIONAL_FN(int, authz_some_auth_required, (request_rec *r));
+APR_DECLARE_OPTIONAL_FN(const char *, authn_ap_auth_type, (request_rec *r));
+APR_DECLARE_OPTIONAL_FN(const char *, authn_ap_auth_name, (request_rec *r));
+
+/* ----------------------------------------------------------------------
+ *
+ * authorization values with mod_access_compat
+ */
+
+APR_DECLARE_OPTIONAL_FN(int, access_compat_ap_satisfies, (request_rec *r));
+
 /* ---------------------------------------------------------------------- */
 
+/** Query the server for some state information
+ * @param query_code Which information is requested
+ * @return the requested state information
+ */
+AP_DECLARE(int) ap_state_query(int query_code);
+
+/*
+ * possible values for query_code in ap_state_query()
+ */
+
+  /** current status of the server */
+#define AP_SQ_MAIN_STATE        0
+  /** are we going to serve requests or are we just testing/dumping config */
+#define AP_SQ_RUN_MODE          1
+    /** generation of the top-level apache parent */
+#define AP_SQ_CONFIG_GEN        2
+
+/*
+ * return values for ap_state_query()
+ */
+
+  /** return value for unknown query_code */
+#define AP_SQ_NOT_SUPPORTED       -1
+
+/* values returned for AP_SQ_MAIN_STATE */
+  /** before the config preflight */
+#define AP_SQ_MS_INITIAL_STARTUP   1
+  /** initial configuration run for setting up log config, etc. */
+#define AP_SQ_MS_CREATE_PRE_CONFIG 2
+  /** tearing down configuration */
+#define AP_SQ_MS_DESTROY_CONFIG    3
+  /** normal configuration run */
+#define AP_SQ_MS_CREATE_CONFIG     4
+  /** running the MPM */
+#define AP_SQ_MS_RUN_MPM           5
+  /** cleaning up for exit */
+#define AP_SQ_MS_EXITING           6
+
+/* values returned for AP_SQ_RUN_MODE */
+  /** command line not yet parsed */
+#define AP_SQ_RM_UNKNOWN           1
+  /** normal operation (server requests or signal server) */
+#define AP_SQ_RM_NORMAL            2
+  /** config test only */
+#define AP_SQ_RM_CONFIG_TEST       3
+  /** only dump some parts of the config */
+#define AP_SQ_RM_CONFIG_DUMP       4
+
 #ifdef __cplusplus
 }
 #endif
 
-#endif	/* !APACHE_HTTP_CORE_H */
+#endif  /* !APACHE_HTTP_CORE_H */
 /** @} */
